#
# zadolbator: simple reminder component for Jabber service
#
# Copyright (C) 2007 Alexander Tsvyashchenko
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

#
# Common code for forms-based and messages-based interface
#
module ModifyEventsCmdCommon
  def ModifyEventsCmdCommon.modify_cals(cals, jid)
    modified_events = 0
    cals.each { |cal|
      cal.events.each { |event|
        new_cal = Icalendar::Calendar.new
        new_cal.add_event(event)

        if not Storage::Event.exists?(event.uid)
          e = Storage::Event.new(:ical => new_cal.to_ical, :owner_id => jid)
          e.id = event.uid
          e.save
          modified_events += 1
        elsif (e = Storage::Event.find(event.uid)).owner_id == jid
          e.ical = new_cal.to_ical
          e.save
          modified_events += 1
        else
          raise "You cannot modify other's events"
        end
      }
    }
    return modified_events
  end
end

#
# Forms-based interface
#
class ModifyEventsCmd

  @@cmd_name = 'modify_events'

  def ModifyEventsCmd.cmd_name
    @@cmd_name
  end

  def process(iq, log)
    cmd = iq.first_element('command')
    owner_id = Storage::User.find(iq.from.strip.to_s).id
    iq.type = :result

    if cmd.attributes['action'] == 'cancel'
      cmd.delete_elements('x')
      cmd.attributes['status'] = 'canceled'
    elsif cmd.attributes['action'] != 'prev' and x = cmd.first_element('x')
      if x.attributes['type'] == 'submit'
        # Check what step are we at
        if x.first_element('field').attributes['var'] == 'Events'
          # First step, let's continue
          inp = Jabber::Dataforms::XDataField.new('Modified Events', :text_multi)
          inp.required = true

          x.each_element('field') { |field|
            if field.attributes['var'] and field.attributes['var'] == 'Events'
              field.each_element('value') { |val|
                log.debug(val)
                Storage::Event.find(:all, :conditions => "(id = '#{val.text}') AND (owner_id = '#{owner_id}')").each { |event|
                  StringIO.new(event.ical).each_line { |line|
                    val = inp.add_element('value')
                    val.text = line.chomp
                  }
                  # Add one empty line for clarity
                  val = inp.add_element('value')
                  val.text = ''
                }
              }
            end
          }

          cmd.attributes['status'] = 'executing'
          x.attributes['type'] == 'form'

          x.delete_elements('field')
          x.delete_elements('instructions')
          x.add(instr = Jabber::Dataforms::XDataInstructions.new)
          instr.text = "Please edit events"
          x.add(inp)
          acts = cmd.add_element('actions')
          acts.attributes['execute'] = 'complete'
          acts.add_element('prev')
          acts.add_element('complete')
        else
          # Second step, saving changes
          cmd.attributes['status'] = 'completed'
          x.attributes['type'] = 'result'
          modified_events = 0

          x.each_element('field') { |field|
            if field.attributes['var'] and field.attributes['var'] == 'Modified Events'
              cals = CommonCmds.parse_cals(field, iq, x)
              if cals
                begin
                  modified_events += ModifyEventsCmdCommon.modify_cals(cals, owner_id)
                rescue => ex
                  CommonCmds.error_msg(x, "Failure when modifying events: #{ex}")
                  return false
                end
              end
            end
          }
          x.delete_elements('field')
          x.delete_elements('instructions')
          x.add(instr = Jabber::Dataforms::XDataInstructions.new)
          instr.text = "Success: #{modified_events} event(s) modified"
          return modified_events > 0
        end
      else
        iq.type = :error
        iq.add_element(Jabber::Error.new('bad-request'))
        return false
      end
    else
      cmd.attributes['status'] = 'executing'
      cmd.add(x = Jabber::Dataforms::XData.new(:form))
      x.add(title = Jabber::Dataforms::XDataTitle.new)
      title.text = 'Modify Events'
      x.add(instr = Jabber::Dataforms::XDataInstructions.new)
      instr.text = 'Please select events to modify'
      CommonCmds.populate_events(x, "owner_id = '#{owner_id}'")
      acts = cmd.add_element('actions')
      acts.attributes['execute'] = 'next'
      acts.add_element('next')
    end
    return false
  end

  def add_cmd(jid, query)
    query.add(Jabber::Discovery::Item::new(jid, 'Modify Events', @@cmd_name))
  end
end

#
# Messages-based interface
#
class ModifyEventsMsgCmd

  @@cmd_name = 'modify'
  @@cmd_help = 'send message with events to modify in iCalendar format'

  def ModifyEventsMsgCmd.cmd_name
    @@cmd_name
  end

  def cmd_name
    @@cmd_name
  end

  def ModifyEventsMsgCmd.cmd_help
    @@cmd_help
  end

  def cmd_help
    @@cmd_help
  end

  def process(msg, log)
    text = StringIO.open(msg.body)
    text.readline # Skip first line - it should be command name
    begin
      cals = Icalendar::parse(text)
    rescue => ex
      msg.body = "Exception while parsing iCalendar input: #{ex}"
      return false
    end

    modified_events = 0

    if cals != nil
      begin
        modified_events += ModifyEventsCmdCommon.modify_cals(cals, Storage::User.find(msg.from.strip.to_s).id)
      rescue => ex
        msg.body = "Error when modifying event: #{ex}"
        return false
      end
    end

    msg.body = "Success: #{modified_events} event(s) modified"
    return modified_events > 0
  end
end