#
# zadolbator: simple reminder component for Jabber service
#
# Copyright (C) 2007 Alexander Tsvyashchenko
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

module RemoveEventsCmdCommon

  def RemoveEventsCmdCommon.remove_events(uids, owner_id)
    removed_events = 0
    Storage::Event.transaction {
      # !!!!! Not really optimal !
      uids.each { |uid|
        cond = "(id = '#{uid}') AND (owner_id = '#{owner_id}')"
          removed_events += Storage::Event.count(:conditions => cond)
          Storage::Event.destroy_all(cond)
      }
    }
    return removed_events
  end

end

class RemoveEventsCmd

  @@cmd_name = 'remove_events'

  def RemoveEventsCmd.cmd_name
    @@cmd_name
  end

  def process(iq, log)
    cmd = iq.first_element('command')
    owner_id = Storage::User.find(iq.from.strip.to_s).id
    iq.type = :result

    if cmd.attributes['action'] == 'cancel'
      cmd.delete_elements('x')
      cmd.attributes['status'] = 'canceled'
    elsif x = cmd.first_element('x')
      if x.attributes['type'] == 'submit'

        cmd.attributes['status'] = 'completed'
        x.attributes['type'] = 'result'
        removed_events = 0

        x.each_element('field') { |field|
          if field.attributes['var'] and field.attributes['var'] == 'Events'
            uids = []
            field.each_element('value') { |val|
              uids << val.text
            }
            begin
              removed_events += RemoveEventsCmdCommon.remove_events(uids, owner_id)
            rescue => ex
              CommonCmds.error_msg(x, "Error when removing event: #{ex}")
              return false
            end
          end
        }
        x.delete_elements('field')
        x.delete_elements('instructions')
        x.add(instr = Jabber::Dataforms::XDataInstructions.new)
        instr.text = "Success: #{removed_events} event(s) removed"
        return removed_events > 0
      else
        iq.type = :error
        iq.add_element(Jabber::Error.new('bad-request'))
        return false
      end
    else
      iq.type = :result
      cmd.attributes['status'] = 'executing'
      cmd.add(x = Jabber::Dataforms::XData.new(:form))
      x.add(title = Jabber::Dataforms::XDataTitle.new)
      title.text = 'Remove Events'
      x.add(instr = Jabber::Dataforms::XDataInstructions.new)
      instr.text = 'Please select events to remove'
      CommonCmds.populate_events(x, "owner_id = '#{owner_id}'")
    end
    return false
  end

  def add_cmd(jid, query)
    query.add(Jabber::Discovery::Item::new(jid, 'Remove Events', @@cmd_name))
  end
end

#
# Messages-based interface
#

class RemoveEventsMsgCmd

  @@cmd_name = 'remove'
  @@cmd_help = 'send message with event UIDs list to remove'

  def RemoveEventsMsgCmd.cmd_name
    @@cmd_name
  end

  def cmd_name
    @@cmd_name
  end

  def RemoveEventsMsgCmd.cmd_help
    @@cmd_help
  end

  def cmd_help
    @@cmd_help
  end

  def process(msg, log)
    text = StringIO.open(msg.body)
    text.readline # Skip first line - it should be command name
    lines = text.readlines
    lines.delete_if { |line| line.chomp.empty? }

    removed_events = 0

    begin
      removed_events = RemoveEventsCmdCommon.remove_events(lines, Storage::User.find(msg.from.strip.to_s).id)
    rescue => ex
      msg.body = "Exception while removing events: #{ex}"
      return false
    end

    msg.body = "Success: #{removed_events} event(s) removed"
    return removed_events > 0
  end
end