#
# zadolbator: simple reminder component for Jabber service
#
# Copyright (C) 2007 Alexander Tsvyashchenko
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU
# General Public License as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not,
# write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

class SubscribeToEventsCmd

  @@cmd_name = 'subscribe_to_events'

  def SubscribeToEventsCmd.cmd_name
    @@cmd_name
  end

  def process(iq, log)
    cmd = iq.first_element('command')
    owner_id = Storage::User.find(iq.from.strip.to_s).id

    if cmd.attributes['action'] == 'cancel'
      iq.type = :result
      cmd.delete_elements('x')
      cmd.attributes['status'] = 'canceled'
    elsif x = cmd.first_element('x')
      if x.attributes['type'] == 'submit'
        iq.type = :result
        cmd.attributes['status'] = 'completed'

        subscribed_events = 0
        user = Storage::User.find(owner_id)

        begin
          Storage::Events_Users.transaction {
            # First step: remove all subscriptions
            user.events.clear
            # Second step: put back those that are specified
            x.each_element('field') { |field|
              if field.attributes['var'] and field.attributes['var'] == 'Events'
                field.each_element('value') { |val|
                  user.events << Storage::Event.find(val.text)
                  subscribed_events += 1
                }
              end
            }
          }
        rescue => ex
          CommonCmds.error_msg(x, "Error when changing subscription: #{ex}")
#          iq.type = :error
#          iq.add_element(Jabber::Error.new('not-acceptable'))
          return false
        end

        x.attributes['type'] = 'result'
        x.delete_elements('field')
        x.delete_elements('instructions')
        x.add(instr = Jabber::Dataforms::XDataInstructions.new)
        instr.text = "Success: subscribed to #{subscribed_events} event(s)"
        return subscribed_events > 0
      else
        iq.type = :error
        iq.add_element(Jabber::Error.new('bad-request'))
        return false
      end
    else
      iq.type = :result
      cmd.attributes['status'] = 'executing'
      cmd.add(x = Jabber::Dataforms::XData.new(:form))
      x.add(title = Jabber::Dataforms::XDataTitle.new)
      title.text = 'Subscribe To Events'
      x.add(instr = Jabber::Dataforms::XDataInstructions.new)
      instr.text = 'Please select events to subscribe to'
      inp = CommonCmds.populate_events(x, nil)
      Storage::User.find(owner_id).events.each { |event|
        val = inp.add_element('value')
        val.text = event.id
      }
    end
    return false
  end

  def add_cmd(jid, query)
    query.add(Jabber::Discovery::Item::new(jid, 'Subscribe To Events', @@cmd_name))
  end
end

#
# Messages-based interface
#

class SubscribeToEventsMsgCmd

  @@cmd_name = 'subscribe'
  @@cmd_help = "send list of events uids to subscribe (prefix uid with '-' to unsubscribe instead) or nothing to get current subscriptions"

  def SubscribeToEventsMsgCmd.cmd_name
    @@cmd_name
  end

  def cmd_name
    @@cmd_name
  end

  def SubscribeToEventsMsgCmd.cmd_help
    @@cmd_help
  end

  def cmd_help
    @@cmd_help
  end

  def process(msg, log)
    text = StringIO.open(msg.body)
    text.readline # Skip first line - it should be command name
    lines = text.readlines
    lines.delete_if { |line| line.chomp.empty? }

    subscribed_events = 0
    unsubscribed_events = 0
    res_text = ''

    begin
      owner_id = Storage::User.find(msg.from.strip.to_s).id
      user = Storage::User.find(owner_id)

      if lines.empty?
        res_text = "\n\nList of current subscriptions:\n"
        user.events.each { |e|
          cals = Icalendar::parse(e.ical)
          # We know only one calendar should be present
          cals.first.events.each { |event|
            if event.dtstart.hour() == 0 and event.dtstart.min() == 0 and event.dtstart.sec() == 0
              date = event.dtstart.strftime('%F')
              res_text += "#{e.id}: #{date} - #{event.summary}\n"
            else
              res_text += "#{e.id}: #{event.dtstart} - #{event.summary}\n"
            end
          }
        }
      else
        Storage::Events_Users.transaction {
          lines.each { |line|
            if line[0..0] == "-"
              user.events.delete(Storage::Event.find(line[1..-1]))
              unsubscribed_events += 1
            else
              user.events << Storage::Event.find(line)
              subscribed_events += 1
            end
          }
        }
      end
    rescue => ex
      msg.body = "Error when performing subscription request: #{ex}"
      return false
    end

    msg.body = "Success: subscribed to #{subscribed_events} event(s), " +
               "unsubscribed from #{unsubscribed_events} event(s)" + res_text
    return (subscribed_events > 0 or unsubscribed_events > 0)
  end
end